package stopAwaitAtimeout;

import java.net.*;
import java.io.*;

public class Client extends Thread{
	private final String STR_ACK = "ACK";
	
	//socket
	Socket socket;
	
	//streams to communicate with server
	private ObjectOutputStream oos;
	private ObjectInputStream ois;
	
	public static void main(String[] args) {
		Thread client = new Client();
		client.start();
	}
	
	public void run() {
		socket = new Socket();
		int iOption;
		
		try {
			//connect to server
			socket.connect((new InetSocketAddress("127.0.0.1", 2007)));
			//get output and input stream
			oos = new ObjectOutputStream(socket.getOutputStream());
			ois = new ObjectInputStream(socket.getInputStream());
			
			//begin main loop
			iOption = getUserInput();
			while (iOption != 4) {
				//sent the question to server
				SendMessage (iOption);
				//receive message 
				System.out.println("Server: " + RecvMessage ());
								
				//read next input
				iOption = getUserInput();
			}
			
			//inform server to quit
			SendMessage (iOption);
			
			//close up
			oos.close();
			ois.close();
			socket.close();
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}	
	
	private int getUserInput() throws IOException{
		int rtn = -1;
		BufferedReader stdin = new BufferedReader (new InputStreamReader(System.in));
				
		System.out.println();
		System.out.println("1: Question 1.");
		System.out.println("2: Question 2.");
		System.out.println("3: Question 3.");
		System.out.println("4: Quit.");
		System.out.println("Please input your selection:");
		rtn = Integer.parseInt(stdin.readLine());
		
		switch (rtn) {
		case 1: 
			System.out.println("Client: Question 1.");
			break;
		case 2: 
			System.out.println("Client: Question 2.");
			break;
		case 3: 
			System.out.println("Client: Question 3.");
			break;
		}
		
		return rtn;
	}
	
	private void SendMessage(int msg) throws IOException{
		boolean fSent = false;
		
		while (fSent == false) {
			//send message
			oos.writeInt(msg);
			//send out immediately
			oos.flush();
			//wait for acknowledgement
			try {
				//check if ack is here
				if (socket.getInputStream().available() == 0) {
				//ack is not here, wait for some time
					sleep(1000);
				}
				//check again
				if (socket.getInputStream().available() > 0) {
					//no check here for simplicity
					ois.readObject();
					//mark fSent as successful
					fSent = true;
				} else {
					System.out.println("Failed to send message. Sending again...");
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				System.exit(1);
			} catch (InterruptedException e) {
				e.printStackTrace();
				System.exit(1);
			}
		}
	}
	
	private String RecvMessage () throws IOException{
		String rtn = "";
		boolean fRecved = false;
		
		try {
			while (fRecved == false) {
				if (socket.getInputStream().available() > 0) {
					//receive message
					rtn = (String) ois.readObject();
					//randomly send ACK to simulate message drop
					if (Math.random() < 0.5) {
						//send acknowledgement
						oos.writeObject(STR_ACK);
						//send out immediately
						oos.flush();
						
						//mark fRecved as successful
						fRecved = true;
					} else {
						System.out.println("Failed to receive message. Receiving again...");
					}
				}
			}
		} catch (ClassNotFoundException e) {
				e.printStackTrace();
				System.exit(1);			
		}
		
		return rtn;
	}
}
