package stopAwaitAtimeout;

import java.net.*;
import java.io.*;

public class Server extends Thread{
	private final String STR_ACK = "ACK";
	
	//socket
	Socket socket;
	
	//streams to communicate with server
	private ObjectOutputStream oos;
	private ObjectInputStream ois;
	
	public static void main(String[] args) {
		Thread server = new Server();
		server.start();		
	}
	
	public void run() {
		ServerSocket ssocket;
		int iOption;
		String strAnswer="";
	
		try {
			//create socket
			ssocket = new ServerSocket();
			//bind to a port
			ssocket.bind(new InetSocketAddress(2007));
			//accept connection from client
			socket = ssocket.accept();
			//get output and input stream
			oos = new ObjectOutputStream(socket.getOutputStream());
			ois = new ObjectInputStream(socket.getInputStream());
			
			//begin main loop
			iOption = RecvMessage();
			while (iOption != 4) {
				//choose the right answer
				switch (iOption) {
				case 1:
					strAnswer = "Answer 1.";
					break;
				case 2:
					strAnswer = "Answer 2.";
					break;
				case 3:
					strAnswer = "Answer 3.";
					break;
				}
				
				//sent the answer to client
				SendMessage (strAnswer);
								
				//read next input
				iOption = RecvMessage();
			}
			
			//close up
			oos.close();
			ois.close();
			socket.close();
			ssocket.close();
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	private void SendMessage(String msg) throws IOException{
		boolean fSent = false;
		
		while (fSent == false) {
			//send message
			oos.writeObject(msg);
			//send out immediately
			oos.flush();
			//wait for acknowledgement
			try {
				//check if ack is here
				if (socket.getInputStream().available() == 0) {
				//ack is not here, wait for some time
					sleep(1000);
				}
				//check again
				if (socket.getInputStream().available() > 0) {
					//no check here for simplicity
					ois.readObject();
					//mark fSent as successful
					fSent = true;					
				} else {
					System.out.println("Failed to send message. Sending again...");
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				System.exit(1);
			} catch (InterruptedException e) {
				e.printStackTrace();
				System.exit(1);
			}
		}
	}
	
	private int RecvMessage () throws IOException{
		int rtn=-1;
		boolean fRecved = false;
		
		while (fRecved == false) {
			if (socket.getInputStream().available() > 0) {
				//receive message
				rtn = ois.readInt();
				//randomly send ACK to simulate message drop
				if (Math.random() < 0.5) {
					//send acknowledgement
					oos.writeObject(STR_ACK);
					//send out immediately
					oos.flush();
					
					//mark fRecved as successful
					fRecved = true;
				} else {
					System.out.println("Failed to receive message. Receiving again...");
				}
			}
		}
		
		return rtn;
	}
}
